class ProjectWBS(PWReportPage):
	def init(self):
		super(ProjectWBS, self).init()
		self.project				= None
		self.intervalUnit			= 0
		self.iterIntervalStartDate	= None
		self.withObjective			= True
		self.withAssignments        = True
		self.showLevel				= 3
		self.contentMode			= 0
		self.iterIndex				= 0
		self.iterTaskIndex			= 0
		self.iterTaskLevelIndex     = 0
		self.iterSubTaskIndex       = 0
		self.withMilestones         = 0
		self.iterTask               = None
		self.iterSubTask            = None
		self.iterPhase				= None
		self.iterLevelTask          = None
		self.isCumulated            = False
		return self

	def showExpectedValues(self):
		return self.contentMode == 0 
		
	def showObjective(self):
		return self.project.itemDescription()


	def masterResources(self):
		sortedMasterResources = self.project.sortedMasterResources()
		return sortedMasterResources



	def topPhases(self):
		result = []
		activities = self.project.visibleSubActivitiesArray()
		startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("flatOrder", True)
		if activities:
			for aAct in activities:
				if aAct.isAssignment() == 0:
					if self.withMilestones == 1:
						result.append(aAct)
					else:
						if aAct.isMilestone() == 0:
							result.append(aAct)
			if result:
				startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("flatOrder", True)
				return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_([startDateDescriptor])
		else:
			return NSArray.array()
	
	def colSize(self):
		if len(self.topPhases()) < 11: 
			return len(self.topPhases())
		else:
			return 10
	
	def showSecond(self):
		return self.showLevel >= 2
		
	def showThird(self):
		return self.showLevel == 3

		
	def topTasksPerLevel(self):
		result = []
		activities = self.iterTask.visibleSubActivitiesArray()

		if activities:
			for aAct in activities:
				if aAct.isAssignment() == 0:
					if self.withMilestones == 1:
						result.append(aAct)
					else:
						if aAct.isMilestone() == 0:
							result.append(aAct)
			if result:
				startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("flatOrder", True)
				return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_([startDateDescriptor])
		else:
			return NSArray.array()


	def topTasksPerPhase(self):
		result = []
		activities = self.iterPhase.visibleSubActivitiesArray()

		if activities:
			for aAct in activities:
				if aAct.isAssignment() == 0:
					if self.withMilestones == 1:
						result.append(aAct)
					else:
						if aAct.isMilestone() == 0:
							result.append(aAct)
			if result:
				startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("flatOrder", True)
				return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_([startDateDescriptor])
		else:
			return NSArray.array()


		
	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def totalRowClass(self):
		if len(self.getActsWithGroups())%2:
			return "even"
		
	def dateFormat(self):
		return "medium/short"


	def cellClassForAmount(self, amount):
		if amount < 0:
			return "negativeBalance"
		else:
			return "positiveBalance"

	def plannedBalanceCellClass(self):
		return self.cellClassForAmount(self.iterPhase.budgetPlannedCostBalance())

	def actualBalanceCellClass(self):
		return self.cellClassForAmount(self.iterPhase.budgetActualCostBalance())

	def plannedTotalBalanceCellClass(self):
		return self.cellClassForAmount(self.project.budgetPlannedCostBalance())

	def actualTotalBalanceCellClass(self):
		return self.cellClassForAmount(self.project.budgetActualCostBalance())



		
	def intervalUnitIsTooSmall(self):
		if self.project.expectedEndDate() and self.project.expectedStartDate():
			projectSeconds = self.project.expectedEndDate().timeIntervalSinceDate_(self.project.expectedStartDate())
			intervalSeconds = MEDuration.durationWithAmount_unit_isFloating_relativeError_(1, self.intervalUnit, 1, 0).roughSeconds()
			numberOfIntervals = projectSeconds / intervalSeconds
			return numberOfIntervals > 300
		else:
			return False
	
	def shouldIncludeStartDate(self, startDate):
		endDate =  MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, startDate, self.project.sharedNSCalendar())
		if self.project.approvedBudgetUndistributedBetweenDate_andDate_(startDate, endDate) > 0.0:
			return True
		if self.project.expectedCostBetweenDate_andDate_(startDate, endDate) > 0.0:
			return True
		return False
	
	def intervalStartDates(self):
		start = MEDuration.startOfUnit_containingDate_calendar_(self.intervalUnit, self.project.expectedStartDate(), self.project.sharedNSCalendar())
		end   = self.project.expectedEndDate()
		
		intervalStart = start
		dates = []
		
		while intervalStart<end:
			if self.shouldIncludeStartDate(intervalStart):
				dates.append(intervalStart)
			intervalStart = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, intervalStart, self.project.sharedNSCalendar())
			
		return dates
		
	def setIterIntervalStartDate_(self, date):
		self.iterIntervalStartDate = date
		self.iterIntervalEndDate = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, date, self.project.sharedNSCalendar())
		
		
	def iterIntervalDescription(self):
		return MEDuration.preciseDescriptionForUnit_atDate_calendar_(self.intervalUnit, self.iterIntervalStartDate, self.project.sharedNSCalendar())

	def iterIntervalCalculcationStartDate(self):
		if self.isCumulated:
			return self.project.expectedStartDate()
		else:
			return self.iterIntervalStartDate


	def iterIntervalExpectedWorkCost(self):
		return self.project.expectedWorkCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)
		
	def iterIntervalTotalCost(self):
		return self.project.expectedCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)

	def iterIntervalPhaseTotalCost(self):
		return self.iterPhase.expectedCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)

	def iterIntervalTaskTotalCost(self):
		return self.iterTask.expectedCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)


	def iterIntervalPhaseBudget(self):
		return self.iterTask.approvedBudgetUndistributedBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)

	def iterIntervalProjectBudget(self):
		return self.project.approvedBudgetUndistributedBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)





